/**
 *                _ _   _               _
 * __      ___ __(_) |_| |_ ___ _ __   | |__  _   _
 * \ \ /\ / / '__| | __| __/ _ \ '_ \  | '_ \| | | |
 *  \ V  V /| |  | | |_| ||  __/ | | | | |_) | |_| |
 *   \_/\_/ |_|  |_|\__|\__\___|_| |_| |_.__/ \__, |
 *                                            |___/
 *  ____                  __     __               _
 * / ___|_   _____ _ __   \ \   / /__   __ _  ___| |
 * \___ \ \ / / _ \ '_ \   \ \ / / _ \ / _` |/ _ \ |
 *  ___) \ V /  __/ | | |   \ V / (_) | (_| |  __/ |
 * |____/ \_/ \___|_| |_|    \_/ \___/ \__, |\___|_|
 *                                     |___/
 * Licensed under the GPLv2 License, Version 2.0 (the "License");
 * Copyright (c) Sven Vogel
 *
 * Compressed version:
 *
 *  type V=(f64,f64);fn t(a:V,b:V,f:fn(
 *  a:f64,b:f64)->f64)->V{(f(a.0,b.0),f
 *  (a.1,b.1))}fn main(){print!("{:?}",
 *  t(t((2.42,2.07),(0.3,3.2),|a,b|{a/b
 *  }),t((1.7,2.3),(0.73,0.42),|a,b|{a*
 *  b}),|a,b|{a-b}))}
 */

fn merge_tuple(a: (f64, f64), b: (f64, f64), f: fn(a: f64, b: f64) -> f64) -> (f64, f64) {
    (f(a.0, b.0), f(a.1, b.1))
}

fn add_tuple(a: (f64, f64), b: (f64, f64)) -> (f64, f64) {
    merge_tuple(a, b, |a, b| a + b)
}

fn sub_tuple(a: (f64, f64), b: (f64, f64)) -> (f64, f64) {
    merge_tuple(a, b, |a, b| a - b)
}

fn mul_tuple(a: (f64, f64), b: (f64, f64)) -> (f64, f64) {
    merge_tuple(a, b, |a, b| a * b)
}

fn div_tuple(a: (f64, f64), b: (f64, f64)) -> (f64, f64) {
    merge_tuple(a, b, |a, b| a / b)
}

fn main() {
    println!(
        "{:#?}",
        sub_tuple(
            div_tuple((2.42, 2.07), (0.3, 3.2)),
            mul_tuple((1.7, 2.3), (0.73, 0.42))
        )
    );
}
